#include <iostream>
#include <memory>
#include <boost/program_options.hpp>
#include <boost/lexical_cast.hpp>
#include "artdaq/DAQdata/Globals.hh"
#include "artdaq/Application/TaskType.hh"
#include "artdaq/Application/EventBuilderApp.hh"
#include "artdaq/Application/LoadParameterSet.hh"
#include "artdaq/ExternalComms/MakeCommanderPlugin.hh"
#include "artdaq-mpich-plugin/Application/MPISentry.hh"
#include "artdaq/BuildInfo/GetPackageBuildInfo.hh"
#include "cetlib_except/exception.h"

int main(int argc, char* argv[])
{
	// initialization

	int const wanted_threading_level{ MPI_THREAD_MULTIPLE };
	MPI_Comm local_group_comm;
	std::unique_ptr<artdaq::MPISentry> mpiSentry;

	try
	{
		mpiSentry.reset(new artdaq::MPISentry(&argc, &argv, wanted_threading_level, artdaq::TaskType::EventBuilderTask, local_group_comm));
	}
	catch (cet::exception& errormsg)
	{
		TLOG_ERROR("EventBuilderMain") << errormsg ;
		TLOG_ERROR("EventBuilderMain") << "MPISentry error encountered in EventBuilderMain; exiting..." ;
		throw errormsg;
	}

	struct Config
	{
		fhicl::TableFragment<artdaq::CommanderInterface::Config> commander_config;
		fhicl::Atom<std::string> application_name{ fhicl::Name{ "application_name" }, fhicl::Comment{ "Name of the application for metrics and logging" }, "EventBuilder" };
		fhicl::TableFragment<artdaq::PortManager::Config> portsConfig; ///< Configuration for artdaq Ports
	};

	fhicl::ParameterSet config = LoadParameterSet<Config>(argc, argv, "EventBuilderMain", "This is the EventBuilder's main executable (MPI version)");
	app_name = config.get<std::string>("application_name", "EventBuilder");
	portMan->UpdateConfiguration(config);
	std::string mf_app_name = artdaq::setMsgFacAppName(app_name, config.get<int>("id"));
	artdaq::configureMessageFacility(mf_app_name.c_str());
	TLOG_DEBUG(app_name + "Main") << "Setting application name to " << mf_app_name ;

	// 23-May-2018, KAB: added lookup of the partition number from the command line arguments.
	if (config.has_key("partition_number"))
	{
		artdaq::Globals::partition_number_ = config.get<int>("partition_number");
	}
	TLOG_DEBUG(app_name + "Main") << "Setting partition number to " << artdaq::Globals::partition_number_;

	TLOG_INFO(app_name + "Main") << "artdaq version " <<
		artdaq::GetPackageBuildInfo::getPackageBuildInfo().getPackageVersion()
		<< ", built " <<
		artdaq::GetPackageBuildInfo::getPackageBuildInfo().getBuildTimestamp() ;

	// create the EventBuilderApp
	my_rank = mpiSentry->rank();
	artdaq::EventBuilderApp evb_app;

	TLOG_DEBUG(app_name + "Main") << "Creating EventBuilder Commander plugin" ;
	auto commander = artdaq::MakeCommanderPlugin(config, evb_app);

	TLOG_INFO(app_name + "Main") << "Running Commmander Server" ;
	commander->run_server();
	artdaq::Globals::CleanUpGlobals();
	TLOG_INFO(app_name + "Main") << "Commandable Server ended, exiting..." ;
}
